#!/usr/bin/env python3
# Copyright (C) 2024 Spearhead Systems SRL

import json
from datetime import datetime, timezone
from cmk.base.plugins.agent_based.agent_based_api.v1 import register, Result, Service, State


def check_state_below(alert_percentages, measured_percent):
    if alert_percentages:
        if alert_percentages[1] <= measured_percent:
            return State.CRIT
        elif alert_percentages[0] <= measured_percent:
            return State.WARN
    return State.OK

def check_state_above(alert_percentages, measured_percent):
    if alert_percentages:
        if alert_percentages[1] >= measured_percent:
            return State.CRIT
        elif alert_percentages[0] >= measured_percent:
            return State.WARN
    return State.OK

# Convert JSON entries into dictionaries indexed by name. We're assuming here
# that the name is unique across AZs and resource groups. If not, add the
# 'location' and 'resource_group' fields in each object to the name.
def parse(string_table):
    lookup = {}

    for json_str in string_table:
        obj = json.loads(json_str)
        name = obj["name"]
        group = obj["resource_group"]
        lookup[f"{name}#{resource_group}"] = obj

    return lookup


# Produce a list of Azure objects for discovery.
def discover(section):
    for name, details in sorted(section.items()):
        yield Service(item=name)


# Given a specific keyvault metric, look it up in the parsed output, and produce
# results on that service based upon the metric's range.
def check_keyvault(item, params, section):
    vault = section.get(item)
    if vault is None:
        return

    metrics = vault["metrics"]

    availability = metrics.get("Availability")
    capacity     = metrics.get("SaturationShoebox")
    latency      = metrics.get("ServiceApiLatency")
    hits         = metrics.get("ServiceApiHit")
    results      = metrics.get("ServiceApiResult")

    alert_availability_percent = params.get("availability")
    alert_capacity_percent     = params.get("capacity")
    alert_latency_milliseconds = params.get("latency")

    if availability:
        check_state_below(alert_availability_percent, availability)
        yield Metric(
            name="availability",
            value=availability,
            levels=alert_availability_percent,
            boundaries=(0, 100)
        )

    if capacity:
        check_state_above(alert_capacity_percent, capacity)
        yield Metric(
            name="capacity",
            value=capacity,
            levels=alert_capacity_percent,
            boundaries=(0, 100)
        )

    if latency:
        check_state_above(alert_latency_milliseconds, latency)
        yield Metric(
            name="latency",
            value=latency,
            levels=alert_latency_milliseconds,
            boundaries=(0, None)
        )

    if hits:
        yield Metric(
            name="hits",
            value=hits,
            boundaries=(0, None)
        )

    if results:
        yield Metric(
            name="results",
            value=results,
            boundaries=(0, None)
        )


# Given a specific firewall metric, look it up in the parsed output, and produce
# results on that service based upon the metric's range.
def check_firewall(item, params, section):
    firewall = section.get(item)
    if firewall is None:
        return

    metrics = vault["metrics"]

    availability = metrics.get("FirewallHealth")
    throughput   = metrics.get("Throughput")
    latency      = metrics.get("FirewallLatencyPng")

    alert_availability_percent = params.get("availability")
    alert_latency_milliseconds = params.get("latency")

    if availability:
        check_state_below(alert_availability_percent, availability)
        yield Metric(
            name="availability",
            value=availability,
            levels=alert_availability_percent,
            boundaries=(0, 100)
        )

    if latency:
        check_state_above(alert_latency_milliseconds, latency)
        yield Metric(
            name="latency",
            value=latency,
            levels=alert_latency_milliseconds,
            boundaries=(0, None)
        )

    if throughput:
        yield Metric(
            name="throughput",
            value=thoughput,
            boundaries=(0, None)
        )


def check_defender(item, params, section):
    yield Result(state=state, summary="Expires in %d days" % remaining_days)


register.agent_section(
    name="azure_keyvault",
    parse_function=parse
)

register.check_plugin(
    name="azure_keyvault",
    service_name="Azure Keyvault Metric %s",

    check_function=check_keyvault,
    check_default_parameters={},
    check_ruleset_name="azure_keyvault",

    discovery_function=discover,
)

register.agent_section(
    name="azure_firewall",
    parse_function=parse
)

register.check_plugin(
    name="azure_firewall",
    service_name="Azure Firewall Metric %s",

    check_function=check_firewall,
    check_default_parameters={},
    check_ruleset_name="azure_firewall",

    discovery_function=discover,
)

register.agent_section(
    name="azure_defender",
    parse_function=parse
)

register.check_plugin(
    name="azure_defender",
    service_name="Azure Defender Alert %s",

    check_function=check_defender,
    check_default_parameters={},
    check_ruleset_name="azure_defender",

    discovery_function=discover,
)
